/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editor;

import java.util.Collection;
import java.util.List;

import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.util.EContentAdapter;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.util.LocalSelectionTransfer;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerDropAdapter;
import org.eclipse.swt.dnd.DND;
import org.eclipse.swt.dnd.DropTargetEvent;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.dnd.TransferData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.ui.extension.base.EditorBase;
import org.fortiss.tooling.kernel.ui.util.DragAndDropUtils;
import org.fortiss.tooling.kernel.ui.util.EObjectSelectionUtils;

/**
 * An editor which only consists of a tree viewer. It uses {@link TreeViewer},
 * which also allows to define columns, i.e., use the tree and table at the same
 * time.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 570713B6F67C37F5BD0948CEBAC276D4
 */
public abstract class TreeViewerEditorBase<T extends EObject> extends
		EditorBase<T> {

	/** Stores the {@link TreeViewer}. */
	private TreeViewer treeViewer;

	/** The adapter used for dealing with changes of the */
	private final Adapter editedObjectChanged = new EContentAdapter() {
		@Override
		public void notifyChanged(Notification notification) {
			super.notifyChanged(notification);
			if (notification.getEventType() != Notification.REMOVING_ADAPTER) {
				treeViewer.refresh(true);
			}
		}
	};

	/** {@inheritDoc} */
	@Override
	public void createPartControl(Composite parent) {
		treeViewer = new TreeViewer(parent);

		setupTreeViewer(treeViewer);

		treeViewer.addDropSupport(
				DND.DROP_COPY | DND.DROP_MOVE | DND.DROP_LINK,
				new Transfer[] { LocalSelectionTransfer.getTransfer() },
				new TreeDropAdapter(treeViewer));

		treeViewer.setInput(editedObject);
		getEditorSite().setSelectionProvider(treeViewer);

		editedObject.eAdapters().add(editedObjectChanged);

		initContextMenu();
	}

	/**
	 * Initializes context menu support.
	 */
	protected void initContextMenu() {
		MenuManager menuManager = new MenuManager("#PopupMenu");
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new IMenuListener() {
			@Override
			public void menuAboutToShow(IMenuManager manager) {
				// TODO: add context menu support
			}
		});
		Menu menu = menuManager.createContextMenu(treeViewer.getTree());
		treeViewer.getTree().setMenu(menu);
		getSite().registerContextMenu(menuManager, treeViewer);
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		editedObject.eAdapters().remove(editedObjectChanged);
		super.dispose();
	}

	/** Returns the elements currently selected in the tree. */
	protected Collection<EObject> getTreeSelection() {
		List<EObject> result = EObjectSelectionUtils
				.getEObjectElements(treeViewer.getSelection());
		if (result.isEmpty()) {
			result.add(editedObject);
		}
		return result;
	}

	/**
	 * This method is used to configure the tree viewer, e.g. set the content
	 * provider, and the label provider.
	 */
	protected abstract void setupTreeViewer(TreeViewer treeViewer);

	/** {@inheritDoc} */
	@Override
	public void setFocus() {
		treeViewer.getTree().setFocus();
	}

	/** The drop adapter for the tree. */
	private class TreeDropAdapter extends ViewerDropAdapter {

		/** Remember the last target. */
		Object lastTarget = null;

		/** Constructor. */
		private TreeDropAdapter(Viewer viewer) {
			super(viewer);
			setExpandEnabled(true);
		}

		/**
		 * {@inheritDoc}
		 * <p>
		 * Hook this method to set a "valid" details field.
		 */
		@Override
		public void dragEnter(DropTargetEvent event) {
			event.detail = DND.DROP_COPY;
			// event.data = LocalSelectionTransfer.getTransfer().getSelection();
			super.dragEnter(event);
		}

		/** {@inheritDoc} */
		@Override
		public boolean validateDrop(Object target, int operation,
				TransferData transferType) {
			if (target == null) {
				target = treeViewer.getInput();
			}
			lastTarget = target;

			if (lastTarget instanceof EObject) {

				EObject dropped = DragAndDropUtils
						.extractDroppedEObject(LocalSelectionTransfer
								.getTransfer().getSelection());

				return IElementCompositorService.INSTANCE.canCompose(
						(EObject) lastTarget, dropped, null);
			}
			return false;
		}

		/** {@inheritDoc} */
		@Override
		public boolean performDrop(Object data) {
			if (lastTarget instanceof EObject) {
				final EObject dropped = DragAndDropUtils
						.extractDroppedEObject(data);
				if (IElementCompositorService.INSTANCE.canCompose(
						(EObject) lastTarget, dropped, null)) {
					TreeViewerEditorBase.this.executeCommand(new Runnable() {
						@Override
						public void run() {
							IElementCompositorService.INSTANCE.compose(
									(EObject) lastTarget, dropped, null);
						}
					});
					return true;
				}
			}
			return false;
		}
	}
}
