/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.NodeEditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.requests.SelectionRequest;
import org.eclipse.gef.tools.ConnectionDragCreationTool;
import org.eclipse.gef.tools.DragEditPartsTracker;
import org.eclipse.swt.graphics.Color;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;

/**
 * The base class for all edit parts which are positioned using x/y coordinates
 * in diagrams. Additionally this class introduces a label figure to be shown on
 * the label layer and a decoration figure to shown on the decoration layer.
 * <p>
 * Sub-classes must implement:
 * <UL>
 * <LI>{@link #createBaseFigure()}
 * <LI>{@link #determineBaseFigureBounds()}
 * <LI>{@link #getForegroundColor()}
 * <LI>{@link #getBackgroundColor()}
 * <LI> {@link #defaultConnectionAnchor()}
 * </UL>
 * 
 * <p>
 * Sub-classes may override:
 * <UL>
 * <LI> {@link #createLabelFigure()}, {@link #determineLabelFigureBounds()},
 * {@link #determineLabelFigureText()}
 * <LI>{@link #createDecorationFigure()},
 * {@link #determineDecorationFigureBounds()}
 * </UL>
 * 
 * 
 * @param <T>
 *            the type of the model element wrapped.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash: D6B33EFB704B0CA151E1C945B5BC48C6
 */
public abstract class PositionedEditPartBase<T extends ILayoutedModelElement>
		extends GraphicalEditPartBase<T> implements NodeEditPart {

	/** The base figure of this positioned edit part. */
	protected IFigure baseFigure;

	/** The label figure. */
	protected Label labelFigure;

	/** The decoration figure. */
	protected IFigure decorationFigure;

	/** Constructor. */
	protected PositionedEditPartBase(T modelElement) {
		super(modelElement);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected final IFigure createFigure() {
		baseFigure = createBaseFigure();

		labelFigure = createLabelFigure();
		if (labelFigure != null) {
			getLayer(ExtendedLayerRootEditPart.LABEL_LAYER).add(labelFigure);
		}

		decorationFigure = createDecorationFigure();
		if (decorationFigure != null) {
			getLayer(ExtendedLayerRootEditPart.DECORATION_LAYER).add(
					decorationFigure);
		}

		return baseFigure;
	}

	/**
	 * Creates the base figure. This figure is created before the label and
	 * decoration figure.
	 */
	protected abstract IFigure createBaseFigure();

	/**
	 * Returns the position and size this edit part's main figure should have in
	 * the layout.
	 */
	protected abstract Rectangle determineBaseFigureBounds();

	/**
	 * Creates the label figure. The default implementation does not create a
	 * figure.
	 */
	protected Label createLabelFigure() {
		return null;
	}

	/**
	 * Returns the position and size of the label figure. The text of the label
	 * has been updated using {@link #determineLabelFigureText()} just before
	 * this method is called. The computation may also use the bounds of the
	 * base figure, since its bounds were determined first. The default
	 * implementation returns an invisible rectangle.
	 */
	protected Rectangle determineLabelFigureBounds() {
		return new Rectangle();
	}

	/**
	 * Determines the text of the label figure during a refresh. This method is
	 * called right before {@link #determineLabelFigureBounds()}. The default
	 * implementation returns an empty string.
	 */
	protected String determineLabelFigureText() {
		return "";
	}

	/**
	 * Creates the decoration figure. The default implementation does not create
	 * a figure.
	 */
	protected IFigure createDecorationFigure() {
		return null;
	}

	/**
	 * Returns the position and size of the decoration figure. The computation
	 * may use the bounds of the base figure or the label figure, since both
	 * bounds were determined first.
	 */
	protected Rectangle determineDecorationFigureBounds() {
		return new Rectangle();
	}

	/**
	 * Returns the foreground color used.
	 */
	protected abstract Color getForegroundColor();

	/**
	 * Returns the background color used.
	 */
	protected abstract Color getBackgroundColor();

	/**
	 * Returns the default connection anchor for this port (does not depend on
	 * type of connection).
	 */
	protected abstract ConnectionAnchor defaultConnectionAnchor();

	/** {@inheritDoc} */
	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();

		if (baseFigure != null) {
			baseFigure.setBounds(determineBaseFigureBounds());
			baseFigure.setForegroundColor(getForegroundColor());
			baseFigure.setBackgroundColor(getBackgroundColor());
		}

		if (labelFigure != null) {
			labelFigure.setText(determineLabelFigureText());
			labelFigure.setBounds(determineLabelFigureBounds());
		}

		if (decorationFigure != null) {
			decorationFigure.setBounds(determineDecorationFigureBounds());
		}
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getSourceConnectionAnchor(
			ConnectionEditPart connection) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getTargetConnectionAnchor(
			ConnectionEditPart connection) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 * <p>
	 * Returns the {@link #defaultConnectionAnchor()}.
	 */
	@Override
	public final ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return defaultConnectionAnchor();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public DragTracker getDragTracker(Request request) {
		if (request instanceof SelectionRequest) {
			SelectionRequest sr = (SelectionRequest) request;
			if (sr.getLastButtonPressed() == 1
					&& (sr.isAltKeyPressed() || sr.isControlKeyPressed())) {
				return new ConnectionDragCreationTool();
			} else if (sr.getLastButtonPressed() == 1) {
				return new DragEditPartsTracker(this);
			}
		}
		return null;
	}
}