/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.ui.internal;

import java.io.IOException;
import java.util.EventObject;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.command.CommandStackListener;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.ui.Saveable;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;
import org.fortiss.tooling.kernel.service.listener.IPersistencyServiceListener;
import org.fortiss.tooling.kernel.ui.extension.data.ContextMenuContextProvider;
import org.fortiss.tooling.kernel.ui.internal.views.NavigatorViewPart;
import org.fortiss.tooling.kernel.ui.service.INavigatorService;
import org.fortiss.tooling.kernel.ui.util.EObjectSelectionUtils;

/**
 * This class implements the {@link INavigatorService} interface.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: 10615C5F91C539B5021A54DC0DAC1AB6
 */
public class NavigatorService implements INavigatorService,
		IPersistencyServiceListener, CommandStackListener {

	/** Stores the navigator view. */
	private NavigatorViewPart navigatorViewPart;

	/** Stores the {@link Saveable}s for the top-level elements. */
	private Map<ITopLevelElement, TopLevelElementSaveable> saveables = new HashMap<ITopLevelElement, NavigatorService.TopLevelElementSaveable>();

	/** Constructor. */
	public NavigatorService() {
		for (ITopLevelElement element : IPersistencyService.INSTANCE
				.getTopLevelElements()) {
			element.addCommandStackListener(this);
			saveables.put(element, new TopLevelElementSaveable(element));
		}
		IPersistencyService.INSTANCE.addTopLevelElementListener(this);
	}

	/** Sets the navigator view. */
	public void setNavigatorViewPart(NavigatorViewPart part) {
		this.navigatorViewPart = part;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isNavigatorView(ContextMenuContextProvider provider) {
		return provider == navigatorViewPart;
	}

	/** Returns the navigator view part. */
	public NavigatorViewPart getNavigatorViewPart() {
		return navigatorViewPart;
	}

	/** {@inheritDoc} */
	@Override
	public ISelection getCurrentSelection() {
		return navigatorViewPart.getSite().getSelectionProvider()
				.getSelection();
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementAdded(ITopLevelElement context) {
		context.addCommandStackListener(this);
		saveables.put(context, new TopLevelElementSaveable(context));
		refresh();
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementRemoved(ITopLevelElement context) {
		context.removeCommandStackListener(this);
		saveables.remove(context);
		refresh();
	}

	/** {@inheritDoc} */
	@Override
	public void topLevelElementChanged(ITopLevelElement element) {
		// ignore this, MarkerService will call refresh()
	}

	/** {@inheritDoc} */
	@Override
	public void commandStackChanged(EventObject event) {
		refresh();
	}

	/** {@inheritDoc} */
	@Override
	public void refresh() {
		navigatorViewPart.refresh();
	}

	/** {@inheritDoc} */
	@Override
	public Saveable[] getSaveables() {
		return saveables.values().toArray(new Saveable[saveables.size()]);
	}

	/** {@inheritDoc} */
	@Override
	public Saveable[] getActiveSaveables() {
		Set<Saveable> result = new HashSet<Saveable>();
		for (EObject selection : EObjectSelectionUtils
				.getCurrentSelectionEObjects()) {
			ITopLevelElement context = IPersistencyService.INSTANCE
					.getTopLevelElementFor(selection);
			if (saveables.get(context) != null) {
				result.add(saveables.get(context));
			}
		}
		return result.toArray(new Saveable[result.size()]);
	}

	/**
	 * Implementation of {@link Saveable}s for {@link ITopLevelElement}s.
	 */
	private static class TopLevelElementSaveable extends Saveable {

		/** Stores the {@link ITopLevelElement}. */
		public final ITopLevelElement context;

		/** Constructor. */
		public TopLevelElementSaveable(ITopLevelElement context) {
			this.context = context;
		}

		/** {@inheritDoc} */
		@Override
		public String getName() {
			return context.getSaveableName();
		}

		/** {@inheritDoc} */
		@Override
		public String getToolTipText() {
			return "";
		}

		/** {@inheritDoc} */
		@Override
		public ImageDescriptor getImageDescriptor() {
			return null;
		}

		/** {@inheritDoc} */
		@Override
		public void doSave(IProgressMonitor monitor) throws CoreException {
			try {
				context.doSave(monitor);
			} catch (IOException e) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"Error during save of " + getName(), e);
			}
		}

		/** {@inheritDoc} */
		@Override
		public boolean isDirty() {
			return context.isDirty();
		}

		/** {@inheritDoc} */
		@Override
		public boolean equals(Object object) {
			return object instanceof TopLevelElementSaveable
					&& ((TopLevelElementSaveable) object).context == context;
		}

		/** {@inheritDoc} */
		@Override
		public int hashCode() {
			return context.hashCode();
		}
	}

	/** {@inheritDoc} */
	@Override
	public void revealModelElement(EObject modelElement) {
		navigatorViewPart.revealModelElement(modelElement);
	}
}
