/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.tooling.kernel.extension.data.IElementCompositionContext;
import org.fortiss.tooling.kernel.extension.data.Prototype;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.service.base.IEObjectAware;

/**
 * Interface for element compositor extensions, i.e. classes which know how to
 * compose and decompose {@link EObject}s.
 * <P>
 * Element compositor extensions are handled by
 * {@link IElementCompositorService}.
 * <P>
 * A typical example is the adding and removal of a node in a graph editor. The
 * compositor adds the node to the graph as a child element.
 * 
 * @param <C>
 *            the meta-model class of the container element handled by this
 *            compositor.
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 4F8B26BA3A47A0A7E09A8E3528BBA598
 */
public interface IElementCompositor<C extends EObject> extends IEObjectAware<C> {
	/**
	 * Returns whether the given container may include the given contained
	 * object.
	 */
	boolean canCompose(C container, EObject contained,
			IElementCompositionContext context);

	/**
	 * Returns whether this compositor is able to compose the given prototype.
	 * While {@link #canCompose(EObject, EObject, IElementCompositionContext)}
	 * considers a concrete composition between the container and contained
	 * element, this method returns this compositor's ability to accept the
	 * given prototype in general.
	 */
	boolean canComposePrototype(Prototype prototype);

	/**
	 * Asks the compositor to insert the contained object into the container.
	 * The contained object may be modified (i.e. renamed, etc.) if necessary.
	 * This method returns a boolean, since user-interaction during compose
	 * might cancel the command.
	 */
	boolean compose(C container, EObject contained,
			IElementCompositionContext context);

	/**
	 * Returns whether the given contained element may be removed from the given
	 * container element.
	 */
	boolean canDecompose(C container, EObject contained,
			IElementCompositionContext context);

	/**
	 * Asks the compositor to remove the contained object from the container.
	 * This method returns a boolean, since user-interaction during compose
	 * might cancel the command.
	 */
	boolean decompose(C container, EObject contained,
			IElementCompositionContext context);
}
